
#include <XInput.h>

const uint8_t abxy_pin = A5;
const uint8_t yL_pin = A1;
const uint8_t xL_pin = A2;
const uint8_t yR_pin = A3;
const uint8_t xR_pin = A4;
const uint8_t dpad_pin = A0;


const uint8_t lt_pin = 13;
const uint8_t lb_pin = 12;
const uint8_t swL_pin = 11;
const uint8_t view_pin = 10;
const uint8_t xbox_pin = 9;
const uint8_t home_pin = 8;
const uint8_t swR_pin = 7;
const uint8_t rb_pin = 6;
const uint8_t rt_pin = 5;

const uint16_t analog_range = 1023;

void setup() 
{
  Serial.begin(9600);
  
  pinMode(lt_pin, INPUT_PULLUP);
  pinMode(lb_pin, INPUT_PULLUP);
  pinMode(swL_pin, INPUT_PULLUP);
  pinMode(view_pin, INPUT_PULLUP);
  pinMode(xbox_pin, INPUT_PULLUP);
  pinMode(home_pin, INPUT_PULLUP);
  pinMode(swR_pin, INPUT_PULLUP);
  pinMode(rt_pin, INPUT_PULLUP);
  pinMode(rb_pin, INPUT_PULLUP);
  
  XInput.setRange(JOY_LEFT, 0, analog_range);
  XInput.setRange(JOY_RIGHT, 0, analog_range);
  XInput.begin();
}

void setTrig(bool button, uint8_t XIControl)
{
  if (button) {
    /* Set 100% */
    XInput.setTrigger(XIControl, 255);
  } else {
    /* Release */
    XInput.setTrigger(XIControl, 0);
  }
}

void parseVoltageDivider(const int value, bool& int1, bool& int2, bool& int3, bool& int4)
{
  if (value > 15 && value < 210)        int1 = true;
  else if (value >= 210 && value < 290) int2 = true;
  else if (value >= 290 && value < 850) int3 = true;
  else if (value >= 850)                int4 = true;
}

void loop() {
  int abxy = analogRead(abxy_pin);
  int dpad = analogRead(dpad_pin);

  /* Right analog stick */
  int xR = analogRead(xR_pin);
  int yR = analogRead(yR_pin);

  /* Left analog stick */
  int xL = analogRead(xL_pin);
  int yL = analogRead(yL_pin);

  /* Buttons */
  bool   lt = !digitalRead(lt_pin);
  bool   lb = !digitalRead(lb_pin);
  bool  swL = !digitalRead(swL_pin);
  bool view = !digitalRead(view_pin);
  bool xbox = !digitalRead(xbox_pin);
  bool  hom = !digitalRead(home_pin);
  bool  swR = !digitalRead(swR_pin);
  bool   rt = !digitalRead(rt_pin);
  bool   rb = !digitalRead(rb_pin);

  /* Init action button values */
  bool a = false;
  bool b = false;
  bool x = false;
  bool y = false;

  /* Init dpad values */
  bool up = false;
  bool down = false;
  bool right = false;
  bool left = false;

  /* Set the state of buttons on digital pins */
  XInput.setButton(BUTTON_START, hom);
  XInput.setButton(BUTTON_BACK, view);
  XInput.setButton(BUTTON_LOGO, xbox);
  XInput.setButton(BUTTON_LB, lb);
  XInput.setButton(BUTTON_RB, rb);
  XInput.setButton(BUTTON_L3, swL);
  XInput.setButton(BUTTON_R3, swR);

  /* 
   *  Emulate triggers with digital buttons
   *  Original Xbox trigger butons are analog, since
   *  I don't have those, set the value to either 100% pressed
   *  or 0% pressed.
   */
  setTrig(lt, TRIGGER_LEFT);
  setTrig(rt, TRIGGER_RIGHT);

  /* Parse voltage divider values for ABXY buttons */
  parseVoltageDivider(abxy, a, b, x, y);
  XInput.setButton(BUTTON_A, a);
  XInput.setButton(BUTTON_B, b);
  XInput.setButton(BUTTON_X, x);
  XInput.setButton(BUTTON_Y, y);

  /* Parse voltage divider values for dpad */
  parseVoltageDivider(dpad, left, up, down, right);
  XInput.setDpad(up, down, left, right);

  /* Set joystick values */
  XInput.setJoystickX(JOY_LEFT, yL, true);
  XInput.setJoystickY(JOY_LEFT, xL, true);
  XInput.setJoystickX(JOY_RIGHT, yR, true);
  XInput.setJoystickY(JOY_RIGHT, xR, true);
  
}
